//Coded by Jacob Seaman 1/10/25
/**goes through every row with a handle (in column B5:B) and checks to see if corresponding row of handle in columnG
 * the time that is inputted there is 4 hours or more before the current datetime, if it is, executes checkout command for handle
 * sheet is type SpreadsheetApp.getActiveSpreadsheet()
 */
function checkTimestampsLocation(sheet) {
  //gets all the handles in the sheet

  //theoretical guarentee @type(Array @type({@type(String) handles, @type(Date) last_updated})
  const allPeopleOnSpreadsheet=_getHandles(sheet);
  //@type(Array @type(String)) representing the handles
  const handles=_allHandlesPastXHours(4,allPeopleOnSpreadsheet);
  //checks out all handle and handles
  handles.forEach(function(handle) {
  checkoutHandle(handle);
  });
}
/**function that gets an inputted sheets handles and last updated sheets
 * assumes handles are B5:B and lasupdated in G5:G
 * @param{sheet} @type{appsheet}
 * @returns a array of records of ranges, handles and astupdated. Should map onto each other for 1:1
 */
function _getHandles(sheet) {
  // Define the range for handles and last updated columns
  const handlesRange = sheet.getRange("B5:B" + sheet.getLastRow());
  const lastUpdatedRange = sheet.getRange("E5:E" + sheet.getLastRow());

  // Get values from the ranges as arrays
  const handlesValues = handlesRange.getValues();
  const lastUpdatedValues = lastUpdatedRange.getValues();

  // Initialize an object to store the records
  const records = [];

  // Iterate through the values until the first empty row is found
  for (let i = 0; i < handlesValues.length; i++) {
    const handle = handlesValues[i][0]; // Access the first element of each row
    const lastUpdated = lastUpdatedValues[i][0];

    // Stop processing if an empty row is encountered
    if (handle === "" || lastUpdated === "") {
      break;
    }

    records.push({ handle: handle, lastUpdated: lastUpdated });
  }

  // Return the array of records
  return records;
}

/**
 * looks through the array and returns all handles that are x hours past the datetime.now()
 * negative numbers will be accepted but treated as 0
 * inputs:
 * handlesUpdated @type(Array @type({@type(String|"") handles, @type(Date|"") last_updated}), 
 * x @Type(number), represents the number of hours we should be checking, accepts decimal
 * returns: @type(Array @type(String)) representing the handles
 */
function _allHandlesPastXHours(x, handlesUpdated) {
  
  // Get the current date and time
  const now = new Date();

  // Calculate the time threshold in milliseconds
  const thresholdMilliseconds = x * 60 * 60 * 1000;

  // Initialize an array to store handles past the threshold
  const handlesPastThreshold = [];

  // Iterate through the array of handle records
  handlesUpdated.forEach(record => {
    const { handle, lastUpdated } = record;

    // Check if both handle and lastUpdated are valid
    if (handle && lastUpdated) {
      const lastUpdatedDate = new Date(lastUpdated);

      // Calculate the time difference
      const timeDifference = now - lastUpdatedDate;

      // Add to the result if the time difference exceeds the threshold
      if (timeDifference > thresholdMilliseconds) {
        handlesPastThreshold.push(handle);
      }
    }
  });

  // Return the array of handles that are past the time threshold
  return handlesPastThreshold;
}

/**inputs a handle and checks them out from both Working and directory (if they exist in either)
 * might need to alter this function to be more generalized
 * input:
 * handle @type(String)
 */
function checkoutHandle(handle){
  checkoutHandleDirectory(handle);
  checkoutHandleLocation(handle);
}

/**checkout for the directory sheet
 * finds the handle in the range("B2:B") (if it exists) and sets its coresponding row -> column AX to false
 * input:
 * @param{handle} @type{String}, could or could not exist in directory
 */
function checkoutHandleDirectory(handle) {
  // Get the active spreadsheet and the directory sheet
  const sheet = SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Directory");

  // Define the range for handles starting from B2
  const handlesRange = sheet.getRange("B2:B" + sheet.getLastRow());
  const handlesValues = handlesRange.getValues();

  // Iterate through the handles to find the matching handle
  for (let i = 0; i < handlesValues.length; i++) {
    if (handlesValues[i][0] === handle) {
      // Set the corresponding row in column AX to false
      const row = i + 2; // Adjust for range starting at row 2
      sheet.getRange("A" + row).setValue(false);
      return; // Exit once the handle is found and updated
    }
  }

  // If the handle was not found, optionally log or handle this case
  Logger.log("Handle not found in the directory: " + handle);
}

/**checkout for the Working sheet
 * finds the handle in the range("B2:B") (if it exists) and sets its coresponding row -> column A to false
 * clears column B,C,D,E,F,G
 * sets column H to "Working"
 * input:
 * handle @type{String}, could or could not exist in directory
 */
function checkoutHandleLocation(handle) {
  // Get the active spreadsheet and the Working sheet
  const sheet = SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Working");

  // Define the range for handles starting from B2
  const handlesRange = sheet.getRange("B2:B" + sheet.getLastRow());
  const handlesValues = handlesRange.getValues();

  // Iterate through the handles to find the matching handle
  for (let i = 0; i < handlesValues.length; i++) {
    if (handlesValues[i][0] === handle) {
      const row = i + 2; // Adjust for range starting at row 2

      // Set column A to false
      sheet.getRange("A" + row).setValue(false);
      //C should never be changed
      // Clear columns D through G
      sheet.getRange("D" + row + ":G" + row).clearContent();
      //clear handle
      sheet.getRange("B" + row).clearContent();

      // Set column H to "Online"
      sheet.getRange("H" + row).setValue("Working");

      return; // Exit once the handle is found and updated
    }
  }

  // Optionally log if the handle is not found
  Logger.log("Handle not found in the Working sheet: " + handle);
}


