var LOGOUT_TIME = 4; // hours
var CLEAR_TIME = 8 // hours
var LIST_START_ROW = 5;
var EST_CHECKOUT_PH= 'Enter Checkout Time';
var NAME_PH='Enter Name';
var GROUP_PH='Pick Group';
let IN_USE_CELL2 = SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Directory").getRange("E2");
let WORKING_REF = SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Working");

/**
 * Finds the last non-empty row in column B of the "Working" sheet.
 * 
 * @returns {number} The last row index that contains data in column B.
 */
function getLastRowInColumnB2() {
  const column = 2; // Column B
  const values = SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Working").getRange(1, column, 163, 1).getValues(); // Get all values in column B
  for (let i = LIST_START_ROW-1; i < 163; i++) {
    if (values[i][0] === "") { // Check for the first empty cell
      Logger.log(i+1);
      return i+1; // Adjust for 1-based row indexing
    }
  }
  Logger.log(0);
  return LIST_START_ROW; // Return 0 if the column is completely empty
}


/**
 * Ensures timestamps and name updates when the checkbox in Column A is toggled.
 * Optimized for performance by reducing API calls and improving reliability.
 * 
 * @param {GoogleAppsScript.Events.SheetsOnEdit} e - The edit event.
 */
function onEdit(e) {
  if (!e) return;

  const lock = LockService.getDocumentLock();
  try {
    if (!lock.tryLock(2000)) return; 

    const sheet = e.source.getActiveSheet();
    const range = e.range;
    const row = range.getRow();
    const col = range.getColumn();

    const isWorking = sheet.getName() === "Working";
    const isDirectory = sheet.getName() === "Directory";
    if (!isWorking && !isDirectory) return;
    // if (!isDirectory && col !== 4 || col !== 5) {
    //   return; 
    // } 

    const CHECKBOX_COL = 1;
    const TIMESTAMP_COL = 3;
    const NAME_COL = 2;
    const PURPOSE_COL = 6;
    const LAST_UPDATED_COL = 7;

    const now = new Date();
    const formattedTime = Utilities.formatDate(now, Session.getScriptTimeZone(), "hh:mm a");

    const spreadsheet = e.source;
    const working = spreadsheet.getSheetByName("Working");
    const directory = spreadsheet.getSheetByName("Directory");
    if (!working || !directory) return;

    let handle = sheet.getRange(row, NAME_COL).getValue().trim();
    let values = sheet.getRange(row, 1, 1, 7).getValues()[0];

    if (isWorking && col === CHECKBOX_COL && range.getValue() === true) {
        const resolvedHandle = getHandleFromEmail();
        if (!handle || handle === "Enter Name") {
          handle = resolvedHandle;
          values[NAME_COL - 1] = handle;
          values[TIMESTAMP_COL - 1] = ""; // Protect the formula (clear this for now)
          values[LAST_UPDATED_COL - 1] = formattedTime;
          values[PURPOSE_COL - 1] = "";

          // Update columns (without formula column)
          // sheet.getRange(row, 1).setValue(true);   // Checkbox A
          sheet.getRange(row, 2).setValue(handle); // Name B
          sheet.getRange(row, 4).clearContent();   // Group D (if needed)
          sheet.getRange(row, 5).setValue(formattedTime); // Timestamp E
          sheet.getRange(row, 6).clearContent();   // Purpose F
          sheet.getRange(row, 7).setValue(formattedTime); // Last Updated G

          // Explicitly reapply formula in C
          sheet.getRange(row, TIMESTAMP_COL).setFormula(`=IF(E${row}="", "", TEXT(E${row}, "hh:mm AM/PM"))`);
      }

      // Sync with Directory after setting values
      const directoryRow = findRowByHandle(directory, handle);
      if (directoryRow !== -1) {
        directory.getRange(directoryRow, CHECKBOX_COL).setValue(true);
      }

      reminderToCloseDoor();
    }

    if (isWorking && col === CHECKBOX_COL && range.getValue() === false) {
      const lastDataRow = findLastDataRow(working);

      // Uncheck logic
      sheet.getRange(row, 1, 1, 7).setValues([[false,"","","","","",""]]);

      // Reset formula after clearing
      sheet.getRange(row, TIMESTAMP_COL).setFormula(`=IF(E${row}="", "", TEXT(E${row}, "hh:mm AM/PM"))`);

      // If not the last data row, move the rest up
      if (lastDataRow !== row) {
        for (let i = row + 1; i <= lastDataRow; i++) {
          const rowData = working.getRange(i, 1, 1, 7).getValues()[0];
          // Move data up one row
          working.getRange(i - 1, 1, 1, 7).setValues([rowData]);
          
          // Clear the row that was moved up
          working.getRange(i, 1, 1, 7).setValues([[false,"","","","","",""]]);
        }
        // Reset formula for the row that was just moved up
        sheet.getRange(i, TIMESTAMP_COL).setFormula(`=IF(E${i}="", "", TEXT(E${i}, "hh:mm AM/PM"))`);
      }

      const directoryRow = findRowByHandle(directory, handle);
      if (directoryRow !== -1) {
        directory.getRange(directoryRow, CHECKBOX_COL).setValue(false);
        if (SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Emails").getRange().getValues("E2:E").includes(working.getRange(row, NAME_COL).getValue().toLowerCase())) {
          Logger.log("User found in email sheet");
        }
      }
    }

    if (isWorking && col === PURPOSE_COL) {
      sheet.getRange(row, LAST_UPDATED_COL).setValue(formattedTime);
    }

    // If triggered from Directory, ensure Working is updated correctly
    if (isDirectory && col === CHECKBOX_COL && range.getValue() === true) {
      // const workingRow = findRowByHandle(working, handle);
      const workingRow = getLastRowInColumnB2(); 
      if (workingRow !== -1) {
        working.getRange(workingRow, NAME_COL).setValue(handle).setFontWeight("bold");
        working.getRange(workingRow, CHECKBOX_COL).setValue(true);
        working.getRange(workingRow, 5).setValue(formattedTime); // Column E
        working.getRange(workingRow, LAST_UPDATED_COL).setValue(formattedTime);

        // Explicitly restore formula in C
        working.getRange(workingRow, TIMESTAMP_COL)
          .setFormula(`=IF(E${workingRow}="", "", TEXT(E${workingRow}, "hh:mm AM/PM"))`);
      }
    }

  } finally {
    lock.releaseLock();
  }
}


/**
 * 🔍 Helper function to find row by name in a sheet.
 */
function findRowByHandle(sheet, handle) {
  const data = sheet.getRange("B:B").getValues().flat(); // Get all names in column B
  for (let i = 1; i < data.length; i++) { // Start from row 2 (skip headers)
    if (data[i] && data[i].toString().trim().toLowerCase() === handle.toString().trim().toLowerCase()) {
      return i + 1; // Convert to 1-based row index
    }
  }
  return -1; // Not found
}

/**
 * Helper function to find the last row with data in the working sheet.
 */
function findLastDataRow(sheet) {
  const lastRow = sheet.getLastRow();
  const data = sheet.getRange(LIST_START_ROW, 2, lastRow - LIST_START_ROW + 1, 1).getValues();
  
  for (let i = 0; i < data.length; i++) {
    if (data[i][0] === "") {
      return i + LIST_START_ROW; // Return the first empty row
    }
  }
  
  return lastRow; // Return the last row if all rows have data
}

function clearDirectory(){
  var handleNotFound = true;
  var startRow = 5;
  while(handleNotFound){
    if(WORKING_REF.getRange(startRow, 1).getValue()){
      WORKING_REF.getRange(startRow, 1).setValue(false);
      clearRow(WORKING_REF, startRow-1);
    } else {
      handleNotFound = false;
    }
  }
}